@@ -158,8 +158,7 @@ shl(register digit *p, register int len, register int sh)
  * leading zeros).
  */
 u64
-__qdivrem(uq, vq, arq)
-	u64 uq, vq, *arq;
+__qdivrem(u64 uq, u64 vq, u64 *arq)
 {
 	union uu tmp;
 	digit *u, *v, *q;
@@ -382,8 +381,7 @@ __divdi3(s64 a, s64 b)
  * Divide two unsigned quads.
  */
 u64
-__udivdi3(a, b)
-        u64 a, b;
+__udivdi3(u64 a, u64 b)
 {
 
         return (__qdivrem(a, b, (u64 *)0));
@@ -394,9 +392,9 @@ __udivdi3(a, b)
  */
 u64 __umoddi3(u64 a, u64 b)
 {
-    u64 rem;
-    __qdivrem(a, b, &rem);
-    return rem;
+	u64 rem;
+	__qdivrem(a, b, &rem);
+	return rem;
 }
 
 /*
@@ -425,19 +423,46 @@ s64 __moddi3(s64 a, s64 b)
 		ub = b, neg2 = 0;
 	__qdivrem(ua, ub, &urem);
     
-    /* There 4 different cases: */
-    if(neg1)
-    {
-        if(neg2)
-            return -urem;
-        else
-            return ub - urem;
-    }   
-    else
-        if(neg2)
-            return -ub + urem;
-        else
-            return urem;
+	/* There 4 different cases: */
+	if (neg1) {
+		if (neg2)
+			return -urem;
+		else
+			return ub - urem;
+	} else {
+		if (neg2)
+			return -ub + urem;
+		else
+			return urem;
+	}
 }
 
 #endif /* BITS_PER_LONG == 32 */
+
+unsigned long long parse_size_and_unit(char *s)
+{
+	unsigned long long ret = simple_strtoull(s, &s, 0);
+
+	switch (*s) {
+	case 'G': case 'g':
+		ret <<= 10;
+	case 'M': case 'm':
+		ret <<= 10;
+	case 'K': case 'k': default:
+		ret <<= 10;
+	case 'B': case 'b':
+		break;
+	}
+
+	return ret;
+}
+
+/*
+ * Local variables:
+ * mode: C
+ * c-set-style: "BSD"
+ * c-basic-offset: 8
+ * tab-width: 8
+ * indent-tabs-mode: t
+ * End:
+ */
