@@ -1,22 +1,17 @@
-/*
- * include/asm-x86/processor.h
- *
- * Copyright (C) 1994 Linus Torvalds
- */
+
+/* Portions are: Copyright (c) 1994 Linus Torvalds */
 
 #ifndef __ASM_X86_PROCESSOR_H
 #define __ASM_X86_PROCESSOR_H
 
 #ifndef __ASSEMBLY__
-#include <asm/page.h>
+#include <xen/config.h>
+#include <xen/cache.h>
+#include <xen/types.h>
+#include <public/xen.h>
 #include <asm/types.h>
 #include <asm/cpufeature.h>
 #include <asm/desc.h>
-#include <asm/flushtlb.h>
-#include <asm/pdb.h>
-#include <xen/config.h>
-#include <xen/spinlock.h>
-#include <public/xen.h>
 #endif
 
 /*
@@ -31,8 +26,7 @@
 #define X86_VENDOR_RISE 6
 #define X86_VENDOR_TRANSMETA 7
 #define X86_VENDOR_NSC 8
-#define X86_VENDOR_SIS 9
-#define X86_VENDOR_NUM 10
+#define X86_VENDOR_NUM 9
 #define X86_VENDOR_UNKNOWN 0xff
 
 /*
@@ -63,6 +57,7 @@
 #define X86_CR0_MP              0x00000002 /* Monitor Coprocessor      (RW) */
 #define X86_CR0_EM              0x00000004 /* Require FPU Emulation    (RO) */
 #define X86_CR0_TS              0x00000008 /* Task Switched            (RW) */
+#define X86_CR0_ET              0x00000010 /* Extension type           (RO) */
 #define X86_CR0_NE              0x00000020 /* Numeric Error Reporting  (RW) */
 #define X86_CR0_WP              0x00010000 /* Supervisor Write Protect (RW) */
 #define X86_CR0_AM              0x00040000 /* Alignment Checking       (RW) */
@@ -84,31 +79,36 @@
 #define X86_CR4_PCE		0x0100	/* enable performance counters at ipl 3 */
 #define X86_CR4_OSFXSR		0x0200	/* enable fast FPU save and restore */
 #define X86_CR4_OSXMMEXCPT	0x0400	/* enable unmasked SSE exceptions */
+#define X86_CR4_VMXE		0x2000  /* enable VMX */
 
 /*
  * Trap/fault mnemonics.
  */
-#define TRAP_divide_error     0
-#define TRAP_debug            1
-#define TRAP_nmi              2
-#define TRAP_int3             3
-#define TRAP_overflow         4
-#define TRAP_bounds           5
-#define TRAP_invalid_op       6
-#define TRAP_no_device        7
-#define TRAP_double_fault     8
-#define TRAP_copro_seg        9
-#define TRAP_invalid_tss     10
-#define TRAP_no_segment      11
-#define TRAP_stack_error     12
-#define TRAP_gp_fault        13
-#define TRAP_page_fault      14
-#define TRAP_spurious_int    15
-#define TRAP_copro_error     16
-#define TRAP_alignment_check 17
-#define TRAP_machine_check   18
-#define TRAP_simd_error      19
-#define TRAP_deferred_nmi    31
+#define TRAP_divide_error      0
+#define TRAP_debug             1
+#define TRAP_nmi               2
+#define TRAP_int3              3
+#define TRAP_overflow          4
+#define TRAP_bounds            5
+#define TRAP_invalid_op        6
+#define TRAP_no_device         7
+#define TRAP_double_fault      8
+#define TRAP_copro_seg         9
+#define TRAP_invalid_tss      10
+#define TRAP_no_segment       11
+#define TRAP_stack_error      12
+#define TRAP_gp_fault         13
+#define TRAP_page_fault       14
+#define TRAP_spurious_int     15
+#define TRAP_copro_error      16
+#define TRAP_alignment_check  17
+#define TRAP_machine_check    18
+#define TRAP_simd_error       19
+#define TRAP_deferred_nmi     31
+
+/* Set for entry via SYSCALL. Informs return code to use SYSRETQ not IRETQ. */
+/* NB. Same as VGCF_IN_SYSCALL. No bits in common with any other TRAP_ defn. */
+#define TRAP_syscall         256
 
 /*
  * Non-fatal fault/trap handlers return an error code to the caller. If the
@@ -119,23 +119,27 @@
 #define EXCRET_not_a_fault 1 /* It was a trap. No instruction replay needed. */
 #define EXCRET_fault_fixed 1 /* It was fault that we fixed: try a replay. */
 
-/*
- * 'trap_bounce' flags values.
- */
+/* 'trap_bounce' flags values */
 #define TBF_EXCEPTION          1
 #define TBF_EXCEPTION_ERRCODE  2
-#define TBF_EXCEPTION_CR2      4
 #define TBF_INTERRUPT          8
 #define TBF_FAILSAFE          16
 
-/*
- * thread.flags values.
- */
-#define TF_failsafe_return 1
+/* 'arch_vcpu' flags values */
+#define _TF_kernel_mode        0
+#define TF_kernel_mode         (1<<_TF_kernel_mode)
+
+/* #PF error code values. */
+#define PFEC_page_present   (1U<<0)
+#define PFEC_write_access   (1U<<1)
+#define PFEC_user_mode      (1U<<2)
+#define PFEC_reserved_bit   (1U<<3)
+#define PFEC_insn_fetch     (1U<<4)
 
 #ifndef __ASSEMBLY__
 
 struct domain;
+struct vcpu;
 
 /*
  * Default implementation of macro that returns current
@@ -148,31 +152,36 @@ struct domain;
   ({ void *pc; __asm__("movl $1f,%0\n1:":"=g" (pc)); pc; })
 #endif
 
-/*
- *  CPU type and hardware bug flags. Kept separately for each CPU.
- *  Members of this structure are referenced in head.S, so think twice
- *  before touching them. [mj]
- */
-
 struct cpuinfo_x86 {
-    __u8    x86;            /* CPU family */
-    __u8    x86_vendor;     /* CPU vendor */
-    __u8    x86_model;
-    __u8    x86_mask;
-    int     cpuid_level;    /* Maximum supported CPUID level, -1=no CPUID */
-    __u32   x86_capability[NCAPINTS];
-    char    x86_vendor_id[16];
-    int     x86_cache_size;  /* in KB - for CPUS that support this call  */
-    int	    x86_clflush_size;
-    int	    x86_tlbsize;     /* number of 4K pages in DTLB/ITLB combined */
-} __attribute__((__aligned__(SMP_CACHE_BYTES)));
+    __u8 x86;		/* CPU family */
+    __u8 x86_vendor;	/* CPU vendor */
+    __u8 x86_model;
+    __u8 x86_mask;
+    char wp_works_ok;	/* It doesn't on 386's */
+    char hlt_works_ok;	/* Problems on some 486Dx4's and old 386's */
+    char hard_math;
+    char rfu;
+    int  cpuid_level;	/* Maximum supported CPUID level, -1=no CPUID */
+    unsigned int x86_capability[NCAPINTS];
+    char x86_vendor_id[16];
+    char x86_model_id[64];
+    int  x86_cache_size;  /* in KB - valid for CPUS which support this call  */
+    int  x86_cache_alignment;	/* In bytes */
+    char fdiv_bug;
+    char f00f_bug;
+    char coma_bug;
+    char pad0;
+    int  x86_power;
+    unsigned char x86_max_cores; /* cpuid returned max cores value */
+    unsigned char booted_cores; /* number of cores as seen by OS */
+    unsigned char apicid;
+} __cacheline_aligned;
 
 /*
  * capabilities of CPUs
  */
 
 extern struct cpuinfo_x86 boot_cpu_data;
-extern struct tss_struct init_tss[NR_CPUS];
 
 #ifdef CONFIG_SMP
 extern struct cpuinfo_x86 cpu_data[];
@@ -182,29 +191,51 @@ extern struct cpuinfo_x86 cpu_data[];
 #define current_cpu_data boot_cpu_data
 #endif
 
-extern char ignore_irq13;
+extern int phys_proc_id[NR_CPUS];
+extern int cpu_core_id[NR_CPUS];
 
 extern void identify_cpu(struct cpuinfo_x86 *);
 extern void print_cpu_info(struct cpuinfo_x86 *);
+extern unsigned int init_intel_cacheinfo(struct cpuinfo_x86 *c);
 extern void dodgy_tsc(void);
 
+#ifdef CONFIG_X86_HT
+extern void detect_ht(struct cpuinfo_x86 *c);
+#else
+static always_inline void detect_ht(struct cpuinfo_x86 *c) {}
+#endif
+
 /*
  * Generic CPUID function
+ * clear %ecx since some cpus (Cyrix MII) do not set or clear %ecx
+ * resulting in stale register contents being returned.
  */
-static inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
+#define cpuid(_op,_eax,_ebx,_ecx,_edx)          \
+    __asm__("cpuid"                             \
+            : "=a" (*(int *)(_eax)),            \
+              "=b" (*(int *)(_ebx)),            \
+              "=c" (*(int *)(_ecx)),            \
+              "=d" (*(int *)(_edx))             \
+            : "0" (_op), "2" (0))
+
+/* Some CPUID calls want 'count' to be placed in ecx */
+static inline void cpuid_count(
+    int op,
+    int count,
+    unsigned int *eax,
+    unsigned int *ebx,
+    unsigned int *ecx,
+    unsigned int *edx)
 {
     __asm__("cpuid"
-            : "=a" (*eax),
-            "=b" (*ebx),
-            "=c" (*ecx),
-            "=d" (*edx)
-            : "0" (op));
+            : "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx)
+            : "0" (op), "c" (count));
 }
 
 /*
  * CPUID functions returning a single datum
  */
-static inline unsigned int cpuid_eax(unsigned int op)
+static always_inline unsigned int cpuid_eax(unsigned int op)
 {
     unsigned int eax;
 
@@ -214,7 +245,7 @@ static inline unsigned int cpuid_eax(unsigned int op)
             : "bx", "cx", "dx");
     return eax;
 }
-static inline unsigned int cpuid_ebx(unsigned int op)
+static always_inline unsigned int cpuid_ebx(unsigned int op)
 {
     unsigned int eax, ebx;
 
@@ -224,7 +255,7 @@ static inline unsigned int cpuid_ebx(unsigned int op)
             : "cx", "dx" );
     return ebx;
 }
-static inline unsigned int cpuid_ecx(unsigned int op)
+static always_inline unsigned int cpuid_ecx(unsigned int op)
 {
     unsigned int eax, ecx;
 
@@ -234,7 +265,7 @@ static inline unsigned int cpuid_ecx(unsigned int op)
             : "bx", "dx" );
     return ecx;
 }
-static inline unsigned int cpuid_edx(unsigned int op)
+static always_inline unsigned int cpuid_edx(unsigned int op)
 {
     unsigned int eax, edx;
 
@@ -246,27 +277,50 @@ static inline unsigned int cpuid_edx(unsigned int op)
 }
 
 
-#define read_cr0() ({ \
-	unsigned long __dummy; \
-	__asm__( \
-		"mov"__OS" %%cr0,%0\n\t" \
-		:"=r" (__dummy)); \
-	__dummy; \
-})
 
-#define write_cr0(x) \
-	__asm__("mov"__OS" %0,%%cr0": :"r" ((unsigned long)x));
+static inline unsigned long read_cr0(void)
+{
+    unsigned long __cr0;
+    __asm__("mov %%cr0,%0\n\t" :"=r" (__cr0));
+    return __cr0;
+} 
 
-#define read_cr4() ({ \
-	unsigned long __dummy; \
-	__asm__( \
-		"mov"__OS" %%cr4,%0\n\t" \
-		:"=r" (__dummy)); \
-	__dummy; \
-})
+static inline void write_cr0(unsigned long val)
+{
+	__asm__("mov %0,%%cr0": :"r" ((unsigned long)val));
+}
+
+static inline unsigned long read_cr2(void)
+{
+    unsigned long __cr2;
+    __asm__("mov %%cr2,%0\n\t" :"=r" (__cr2));
+    return __cr2;
+}
+
+static inline unsigned long read_cr4(void)
+{
+    unsigned long __cr4;
+    __asm__("mov %%cr4,%0\n\t" :"=r" (__cr4));
+    return __cr4;
+} 
+    
+static inline void write_cr4(unsigned long val)
+{
+	__asm__("mov %0,%%cr4": :"r" ((unsigned long)val));
+}
+
+
+/* Clear and set 'TS' bit respectively */
+static inline void clts(void) 
+{
+    __asm__ __volatile__ ("clts");
+}
+
+static inline void stts(void) 
+{
+    write_cr0(X86_CR0_TS|read_cr0());
+}
 
-#define write_cr4(x) \
-	__asm__("mov"__OS" %0,%%cr4": :"r" ((unsigned long)x));
 
 /*
  * Save the cr4 feature set we're using (ie
@@ -276,24 +330,26 @@ static inline unsigned int cpuid_edx(unsigned int op)
  */
 extern unsigned long mmu_cr4_features;
 
-static inline void set_in_cr4 (unsigned long mask)
+static always_inline void set_in_cr4 (unsigned long mask)
 {
+    unsigned long dummy;
     mmu_cr4_features |= mask;
-    __asm__("mov"__OS" %%cr4,%%"__OP"ax\n\t"
-            "or"__OS" %0,%%"__OP"ax\n\t"
-            "mov"__OS" %%"__OP"ax,%%cr4\n"
-            : : "irg" (mask)
-            :"ax");
+    __asm__ __volatile__ (
+        "mov %%cr4,%0\n\t"
+        "or %1,%0\n\t"
+        "mov %0,%%cr4\n"
+        : "=&r" (dummy) : "irg" (mask) );
 }
 
-static inline void clear_in_cr4 (unsigned long mask)
+static always_inline void clear_in_cr4 (unsigned long mask)
 {
+    unsigned long dummy;
     mmu_cr4_features &= ~mask;
-    __asm__("mov"__OS" %%cr4,%%"__OP"ax\n\t"
-            "and"__OS" %0,%%"__OP"ax\n\t"
-            "mov"__OS" %%"__OP"ax,%%cr4\n"
-            : : "irg" (~mask)
-            :"ax");
+    __asm__ __volatile__ (
+        "mov %%cr4,%0\n\t"
+        "and %1,%0\n\t"
+        "mov %0,%%cr4\n"
+        : "=&r" (dummy) : "irg" (~mask) );
 }
 
 /*
@@ -327,15 +383,32 @@ static inline void clear_in_cr4 (unsigned long mask)
 	outb((data), 0x23); \
 } while (0)
 
-#define IOBMP_BYTES             8192
-#define IOBMP_BYTES_PER_SELBIT  (IOBMP_BYTES / 64)
-#define IOBMP_BITS_PER_SELBIT   (IOBMP_BYTES_PER_SELBIT * 8)
-#define IOBMP_OFFSET            offsetof(struct tss_struct, io_bitmap)
-#define IOBMP_INVALID_OFFSET    0x8000
+/* Stop speculative execution */
+static inline void sync_core(void)
+{
+    int tmp;
+    asm volatile("cpuid" : "=a" (tmp) : "0" (1) : "ebx","ecx","edx","memory");
+}
 
-struct i387_state {
-    u8 state[512]; /* big enough for FXSAVE */
-} __attribute__ ((aligned (16)));
+static always_inline void __monitor(const void *eax, unsigned long ecx,
+		unsigned long edx)
+{
+	/* "monitor %eax,%ecx,%edx;" */
+	asm volatile(
+		".byte 0x0f,0x01,0xc8;"
+		: :"a" (eax), "c" (ecx), "d"(edx));
+}
+
+static always_inline void __mwait(unsigned long eax, unsigned long ecx)
+{
+	/* "mwait %eax,%ecx;" */
+	asm volatile(
+		".byte 0x0f,0x01,0xc9;"
+		: :"a" (eax), "c" (ecx));
+}
+
+#define IOBMP_BYTES             8192
+#define IOBMP_INVALID_OFFSET    0x8000
 
 struct tss_struct {
     unsigned short	back_link,__blh;
@@ -372,166 +445,40 @@ struct tss_struct {
     u16 trace;
 #endif
     u16 bitmap;
-    u8  io_bitmap[IOBMP_BYTES+1];
-    /* Pads the TSS to be cacheline-aligned (total size is 0x2080). */
-    u8 __cacheline_filler[23];
-};
-
-struct trap_bounce {
-    unsigned long  error_code;
-    unsigned long  cr2;
-    unsigned short flags; /* TBF_ */
-    unsigned short cs;
-    unsigned long  eip;
-};
-
-struct thread_struct {
-    unsigned long      guestos_sp;
-    unsigned long      guestos_ss;
-
-    unsigned long      flags; /* TF_ */
-
-    /* Hardware debugging registers */
-    unsigned long      debugreg[8];  /* %%db0-7 debug registers */
-
-    /* floating point info */
-    struct i387_state  i387;
-
-    /* general user-visible register state */
-    execution_context_t user_ctxt;
-
-    void (*schedule_tail) (struct domain *);
-
-    /*
-     * Return vectors pushed to us by guest OS.
-     * The stack frame for events is exactly that of an x86 hardware interrupt.
-     * The stack frame for a failsafe callback is augmented with saved values
-     * for segment registers %ds, %es, %fs and %gs:
-     * 	%ds, %es, %fs, %gs, %eip, %cs, %eflags [, %oldesp, %oldss]
-     */
-    unsigned long event_selector;    /* 08: entry CS  */
-    unsigned long event_address;     /* 12: entry EIP */
-
-    unsigned long failsafe_selector; /* 16: entry CS  */
-    unsigned long failsafe_address;  /* 20: entry EIP */
-
-    /* Bounce information for propagating an exception to guest OS. */
-    struct trap_bounce trap_bounce;
-
-    /* I/O-port access bitmap. */
-    u64 io_bitmap_sel; /* Selector to tell us which part of the IO bitmap are
-                        * "interesting" (i.e. have clear bits) */
-    u8 *io_bitmap; /* Pointer to task's IO bitmap or NULL */
-
-    /* Trap info. */
-#ifdef __i386__
-    int                fast_trap_idx;
-    struct desc_struct fast_trap_desc;
-#endif
-    trap_info_t        traps[256];
-};
+    /* Pads the TSS to be cacheline-aligned (total size is 0x80). */
+    u8 __cacheline_filler[24];
+} __cacheline_aligned __attribute__((packed));
 
 #define IDT_ENTRIES 256
-extern struct desc_struct idt_table[];
-extern struct desc_struct *idt_tables[];
+extern idt_entry_t idt_table[];
+extern idt_entry_t *idt_tables[];
 
-#if defined(__i386__)
+extern struct tss_struct init_tss[NR_CPUS];
 
-#define SET_DEFAULT_FAST_TRAP(_p) \
-    (_p)->fast_trap_idx = 0x20;   \
-    (_p)->fast_trap_desc.a = 0;   \
-    (_p)->fast_trap_desc.b = 0;
+#ifdef CONFIG_X86_32
 
-#define CLEAR_FAST_TRAP(_p) \
-    (memset(idt_tables[smp_processor_id()] + (_p)->fast_trap_idx, \
-     0, 8))
+extern void init_int80_direct_trap(struct vcpu *v);
+#define set_int80_direct_trap(_ed)                  \
+    (memcpy(idt_tables[(_ed)->processor] + 0x80,    \
+            &((_ed)->arch.int80_desc), 8))
 
-#ifdef XEN_DEBUGGER
-#define SET_FAST_TRAP(_p)   \
-    (pdb_initialized ? (void *) 0 : \
-       (memcpy(idt_tables[smp_processor_id()] + (_p)->fast_trap_idx, \
-               &((_p)->fast_trap_desc), 8)))
 #else
-#define SET_FAST_TRAP(_p)   \
-    (memcpy(idt_tables[smp_processor_id()] + (_p)->fast_trap_idx, \
-            &((_p)->fast_trap_desc), 8))
-#endif
 
-long set_fast_trap(struct domain *p, int idx);
+#define init_int80_direct_trap(_ed) ((void)0)
+#define set_int80_direct_trap(_ed)  ((void)0)
 
 #endif
 
-#define INIT_THREAD { 0 }
+extern int gpf_emulate_4gb(struct cpu_user_regs *regs);
 
-extern int gpf_emulate_4gb(struct xen_regs *regs);
+extern void write_ptbase(struct vcpu *v);
 
-struct mm_struct {
-    /*
-     * Every domain has a L1 pagetable of its own. Per-domain mappings
-     * are put in this table (eg. the current GDT is mapped here).
-     */
-    l1_pgentry_t *perdomain_pt;
-    pagetable_t  pagetable;
-
-    /* shadow mode status and controls */
-    unsigned int shadow_mode;  /* flags to control shadow table operation */
-    pagetable_t  shadow_table;
-    spinlock_t   shadow_lock;
-    unsigned int shadow_max_page_count; // currently unused
-
-    /* shadow hashtable */
-    struct shadow_status *shadow_ht;
-    struct shadow_status *shadow_ht_free;
-    struct shadow_status *shadow_ht_extras; /* extra allocation units */
-    unsigned int shadow_extras_count;
-
-    /* shadow dirty bitmap */
-    unsigned long *shadow_dirty_bitmap;
-    unsigned int shadow_dirty_bitmap_size;  /* in pages, bit per page */
-
-    /* shadow mode stats */
-    unsigned int shadow_page_count;     
-    unsigned int shadow_fault_count;     
-    unsigned int shadow_dirty_count;     
-    unsigned int shadow_dirty_net_count;     
-    unsigned int shadow_dirty_block_count;     
-
-    /* Current LDT details. */
-    unsigned long ldt_base, ldt_ents, shadow_ldt_mapcnt;
-    /* Next entry is passed to LGDT on domain switch. */
-    char gdt[10]; /* NB. 10 bytes needed for x86_64. Use 6 bytes for x86_32. */
-};
-
-static inline void write_ptbase(struct mm_struct *mm)
-{
-    unsigned long pa;
-
-    if ( unlikely(mm->shadow_mode) )
-        pa = pagetable_val(mm->shadow_table);
-    else
-        pa = pagetable_val(mm->pagetable);
-
-    write_cr3(pa);
-}
-
-#define IDLE0_MM                                                    \
-{                                                                   \
-    perdomain_pt: 0,                                                \
-    pagetable:   mk_pagetable(__pa(idle_pg_table))                  \
-}
-
-/* Convenient accessor for mm.gdt. */
-#define SET_GDT_ENTRIES(_p, _e) ((*(u16 *)((_p)->mm.gdt + 0)) = (((_e)<<3)-1))
-#define SET_GDT_ADDRESS(_p, _a) ((*(unsigned long *)((_p)->mm.gdt + 2)) = (_a))
-#define GET_GDT_ENTRIES(_p)     (((*(u16 *)((_p)->mm.gdt + 0))+1)>>3)
-#define GET_GDT_ADDRESS(_p)     (*(unsigned long *)((_p)->mm.gdt + 2))
-
-void destroy_gdt(struct domain *d);
-long set_gdt(struct domain *d, 
+void destroy_gdt(struct vcpu *d);
+long set_gdt(struct vcpu *d, 
              unsigned long *frames, 
              unsigned int entries);
 
-long set_debugreg(struct domain *p, int reg, unsigned long value);
+long set_debugreg(struct vcpu *p, int reg, unsigned long value);
 
 struct microcode_header {
     unsigned int hdrver;
@@ -569,7 +516,7 @@ struct extended_sigtable {
 };
 
 /* REP NOP (PAUSE) is a good thing to insert into busy-wait loops. */
-static inline void rep_nop(void)
+static always_inline void rep_nop(void)
 {
     __asm__ __volatile__ ( "rep;nop" : : : "memory" );
 }
@@ -580,7 +527,7 @@ static inline void rep_nop(void)
 #ifdef 	CONFIG_MPENTIUMIII
 
 #define ARCH_HAS_PREFETCH
-extern inline void prefetch(const void *x)
+extern always_inline void prefetch(const void *x)
 {
     __asm__ __volatile__ ("prefetchnta (%0)" : : "r"(x));
 }
@@ -591,12 +538,12 @@ extern inline void prefetch(const void *x)
 #define ARCH_HAS_PREFETCHW
 #define ARCH_HAS_SPINLOCK_PREFETCH
 
-extern inline void prefetch(const void *x)
+extern always_inline void prefetch(const void *x)
 {
     __asm__ __volatile__ ("prefetch (%0)" : : "r"(x));
 }
 
-extern inline void prefetchw(const void *x)
+extern always_inline void prefetchw(const void *x)
 {
     __asm__ __volatile__ ("prefetchw (%0)" : : "r"(x));
 }
@@ -604,12 +551,41 @@ extern inline void prefetchw(const void *x)
 
 #endif
 
-void show_guest_stack();
-void show_trace(unsigned long *esp);
-void show_stack(unsigned long *esp);
-void show_registers(struct xen_regs *regs);
-asmlinkage void fatal_trap(int trapnr, struct xen_regs *regs);
+void show_stack(struct cpu_user_regs *regs);
+void show_xen_trace(void);
+void show_stack_overflow(unsigned long esp);
+void show_registers(struct cpu_user_regs *regs);
+void show_execution_state(struct cpu_user_regs *regs);
+void show_page_walk(unsigned long addr);
+asmlinkage void fatal_trap(int trapnr, struct cpu_user_regs *regs);
+
+/* Dumps current register and stack state. */
+#define dump_execution_state()                                              \
+    /* NB. Needs interrupts enabled else we end up in fatal_trap(). */      \
+    __asm__ __volatile__ ( "pushf ; sti ; ud2 ; .ascii \"dbg\" ; popf" )
+
+extern void mtrr_ap_init(void);
+extern void mtrr_bp_init(void);
+
+extern void mcheck_init(struct cpuinfo_x86 *c);
+
+int cpuid_hypervisor_leaves(
+    uint32_t idx, uint32_t *eax, uint32_t *ebx, uint32_t *ecx, uint32_t *edx);
+int rdmsr_hypervisor_regs(
+    uint32_t idx, uint32_t *eax, uint32_t *edx);
+int wrmsr_hypervisor_regs(
+    uint32_t idx, uint32_t eax, uint32_t edx);
 
 #endif /* !__ASSEMBLY__ */
 
 #endif /* __ASM_X86_PROCESSOR_H */
+
+/*
+ * Local variables:
+ * mode: C
+ * c-set-style: "BSD"
+ * c-basic-offset: 4
+ * tab-width: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
